@@ -173,27 +173,11 @@ sub max_id {
 }
 
 #
-# Returns a hash of paginated items that belong to the xpath.
+# Returns a hash of paginated items
 # 
-sub paginate_xpath {
-    my( $self, $path, $paginate_length, $paginate_start ) = @_;
+sub paginate_hash {
+    my( $self, $obj_id, $paginate_length, $paginate_start ) = @_;
 
-    my( @list ) = _xpath_to_list( $path );
-    my $next_ref = 1;
-    for my $l (@list) {
-        next if $l eq ''; #skip blank paths like /foo//bar/  (should just look up foo -> bar
-
-        my( $val, $ref ) = $self->_selectrow_array( "SELECT value, ref_id FROM field WHERE field=? AND obj_id=?",  $l, $next_ref );
-        die $self->{DBH}->errstr() if $self->{DBH}->errstr();
-
-
-        if( $ref ) {
-            $next_ref = $ref;
-        }
-        else {
-	    die "Unable to find xpath location '$path' for pagination";
-        }
-    } #each path part
 
     my $PAG = '';
     if( defined( $paginate_start ) ) {
@@ -203,39 +187,23 @@ sub paginate_xpath {
 	}
     }    
 
-    my $res = $self->_selectall_arrayref( "SELECT field, ref_id, value FROM field WHERE obj_id=? ORDER BY field $PAG", $next_ref );
-#    return [ map { [ $_->[0], $_->[1] || 'v' . $_->[2] ] } @$res ];
+    my $res = $self->_selectall_arrayref( "SELECT field, ref_id, value FROM field WHERE obj_id=? ORDER BY field $PAG", $obj_id );
+
     my %ret;
     for my $row (@$res) {
 	$ret{$row->[0]} = $row->[1] || "v$row->[2]";
     }
     return \%ret
-} #paginate_xpath
+} #paginate_hash
+
 
 #
-# Returns a hash of paginated items that belong to the xpath. Note that this 
+# Returns a hash of paginated items that belong to the list. Note that this 
 # does not preserve indexes ( for example, if the list has two rows, and first index in the database is 3, the list returned is still [ 'val1', 'val2' ]
 #   rather than [ undef, undef, undef, 'val1', 'val2' ]
 #
-sub paginate_xpath_list {
-    my( $self, $path, $paginate_length, $paginate_start, $reverse ) = @_;
-
-    my( @list ) = _xpath_to_list( $path );
-    my $next_ref = 1;
-    for my $l (@list) {
-        next if $l eq ''; #skip blank paths like /foo//bar/  (should just look up foo -> bar
-
-        my( $val, $ref ) = $self->_selectrow_array( "SELECT value, ref_id FROM field WHERE field=? AND obj_id=?",  $l, $next_ref );
-        die $self->{DBH}->errstr() if $self->{DBH}->errstr();
-
-
-        if( $ref ) {
-            $next_ref = $ref;
-        }
-        else {
-	    die "Unable to find xpath location '$path' for pagination";
-        }
-    } #each path part
+sub paginate_list {
+    my( $self, $obj_id, $paginate_length, $paginate_start, $reverse ) = @_;
 
     my $PAG = '';
     if( defined( $paginate_length ) ) {
@@ -246,51 +214,14 @@ sub paginate_xpath_list {
 	}
     }    
 
-    my $res = $self->_selectall_arrayref( "SELECT field, ref_id, value FROM field WHERE obj_id=? ORDER BY cast( field as int ) " . ( $reverse ? 'DESC ' : '' ) . " $PAG", $next_ref );
+    my $res = $self->_selectall_arrayref( "SELECT field, ref_id, value FROM field WHERE obj_id=? ORDER BY cast( field as int )" .
+					  ( $reverse ? 'DESC ' : '' ) . " $PAG", $obj_id );
     my @ret;
     for my $row (@$res) {
 	push @ret, $row->[1] || "v$row->[2]";
     }
     return \@ret
-} #paginate_xpath_list
-
-#
-# Return a path to root that this object has (specified by id), if any.
-#
-sub path_to_root {
-    my( $self, $obj_id ) = @_;
-    return '' if $obj_id == 1;
-    my $res = $self->_selectall_arrayref( "SELECT obj_id,field FROM field WHERE ref_id=?", $obj_id );
-    for my $row (@$res) {
-	my( $new_obj_id, $field ) = @$row;
-	if( $self->has_path_to_root( $new_obj_id, { $obj_id => 1 } ) ) {
-	    return $self->path_to_root( $new_obj_id ) . "/$field";
-	}
-    }
-
-    return;
-} #path_to_root
-
-#
-# Return all paths to root that this object (specified by id) has, if any.
-#
-sub paths_to_root {
-    my( $self, $obj_id, $seen ) = @_;
-    $seen ||= {};
-    return [''] if $obj_id == 1;
-    my $ret = [];
-    my $res = $self->_selectall_arrayref( "SELECT obj_id,field FROM field WHERE ref_id=?", $obj_id );
-    for my $row (@$res) {
-	my( $new_obj_id, $field ) = @$row;
-	if(  ! $seen->{$new_obj_id} && $self->has_path_to_root( $new_obj_id ) ) {
-	    $seen->{$new_obj_id} = 1;
-	    my $paths = $self->paths_to_root( $new_obj_id, $seen );
-	    push @$ret, map { $_. "/$field" } @$paths;
-	}
-    }
-    
-    return $ret;
-} #paths_to_root
+} #paginate_list
 
 #
 # Finds objects not connected to the root and recycles them.
@@ -394,158 +325,75 @@ sub _engage_queries {
     }
 } #_engage_queries
 
-
 #
-# Returns a single value given the xpath (notation is slash separated from root)
-# This will always query persistance directly for the value, bypassing objects.
-# The use for this is to fetch specific things from potentially very long hashes that you don't want to
-#   load in their entirety.
+# Returns the number of entries in the list of the given id.
 #
-sub xpath {
-    my( $self, $path ) = @_;
-    my( @list ) = _xpath_to_list( $path );
-    my $next_ref = 1;
-    my $final_val;
-    for my $l (@list) {
-        next if $l eq ''; #skip blank paths like /foo//bar/  (should just look up foo -> bar
-        undef $final_val;
-        my( $val, $ref ) = $self->_selectrow_array( "SELECT value, ref_id FROM field WHERE field=? AND obj_id=?",  $l, $next_ref );
-        die $self->{DBH}->errstr() if $self->{DBH}->errstr();
-
-        if( $ref ) {
-            $next_ref = $ref;
-            $final_val = $ref;
-        }
-	else {
-            $final_val = "v$val";
-            last;
-        }
-    } #each path part
-
-    # @TODO: log bad xpath if final_value not defined
-
-    return $final_val;
-} #xpath
-
-#
-# Returns the number of entries in the data structure given.
-#
-sub xpath_count {
-    my( $self, $path ) = @_;
-    my( @list ) = _xpath_to_list( $path );
-    my $next_ref = 1;
-    for my $l (@list) {
-        next if $l eq ''; #skip blank paths like /foo//bar/  (should just look up foo -> bar
-
-        my( $ref ) = $self->_selectrow_array( "SELECT ref_id FROM field WHERE field=? AND obj_id=?",  $l, $next_ref );
-        die $self->{DBH}->errstr() if $self->{DBH}->errstr();
-
-        $next_ref = $ref;
-        last unless $next_ref;
-    } #each path part
-
-    my( $count ) = $self->_selectrow_array( "SELECT count(*) FROM field WHERE obj_id=?",  $next_ref );
+sub count {
+    my( $self, $container_id ) = @_;
+    my( $count ) = $self->_selectrow_array( "SELECT count(*) FROM field WHERE obj_id=?",  $container_id );
     die $self->{DBH}->errstr() if $self->{DBH}->errstr();
 
-
     return $count;
+} #count
 
-} #xpath_count
-
-
-#
-# Deletes a value from the given xpath. /foo/bar/baz.
-#
-sub xpath_delete {
-    my( $self, $path ) = @_;
-
-    my( @list ) = _xpath_to_list( $path );
-    my $field = pop @list;
-    my $next_ref = 1;
-    for my $l (@list) {
-        next if $l eq ''; #skip blank paths like /foo//bar/  (should just look up foo -> bar
-
-        my( $val, $ref ) = $self->_selectrow_array( "SELECT value, ref_id FROM field WHERE field=? AND obj_id=?",  $l, $next_ref );
-        die $self->{DBH}->errstr() if $self->{DBH}->errstr();
-
-        if( $ref ) {
-            $next_ref = $ref;
-        }
-        else {
-	    die "Unable to find xpath location '$path' for delete";
-        }
-    } #each path part
-
-    #find the object type to see if this is an array to append to, or a hash to insert to
-    $self->_do( "DELETE FROM field WHERE obj_id = ? AND field=?", $next_ref, $field );
-
-} #xpath_delete
-
-#
-# Inserts a value into the given xpath. /foo/bar/baz. Overwrites old value if it exists.
-#
-sub xpath_insert {
-    my( $self, $path, $item_to_insert ) = @_;
-
-    my( @list ) = _xpath_to_list( $path );
-    my $field = pop @list;
-    my $next_ref = 1;
-    for my $l (@list) {
-        next if $l eq ''; #skip blank paths like /foo//bar/  (should just look up foo -> bar
-
-        my( $val, $ref ) = $self->_selectrow_array( "SELECT value, ref_id FROM field WHERE field=? AND obj_id=?",  $l, $next_ref );
-        die $self->{DBH}->errstr() if $self->{DBH}->errstr();
-
-        if( $ref ) {
-            $next_ref = $ref;
-        }
-        else {
-	    die "Unable to find xpath location '$path' for insert";
-        }
-    } #each path part
-
-    $self->_do( "DELETE FROM field WHERE obj_id = ? AND field=?", $next_ref, $field );
-
-    if( index( $item_to_insert, 'v' ) == 0 ) {
-	$self->_do( "INSERT INTO field (obj_id,field,value) VALUES (?,?,?)", $next_ref, $field, substr( $item_to_insert, 1)  );
+sub list_insert {
+    my( $self, $list_id, $val, $idx ) = @_;
+    unless( defined( $idx ) ) {
+	( $idx ) = $self->{DBH}->selectrow_array( "SELECT 1 + max( field ) FROM field WHERE obj_id=?", {}, $list_id );
     } else {
-	$self->_do( "INSERT INTO field (obj_id,field,ref_id) VALUES (?,?,?)", $next_ref, $field, $item_to_insert );
+	my( $occupied ) = $self->{DBH}->selectrow_array( "SELECT count(*) FROM field WHERE obj_id=? AND field=?", {}, $list_id, $idx );
+	if( $occupied ) {
+	    $self->{DBH}->selectrow_array( "UPDATE field SET field=field+1 WHERE obj_id=? AND field >= ?", {}, $list_id, $idx );
+	}
     }
-
-} #xpath_insert
-
-#
-# Appends a value into the list located at the given xpath.
-#
-sub xpath_list_insert {
-    my( $self, $path, $item_to_insert ) = @_;
-
-    my( @list ) = _xpath_to_list( $path );
-    my $next_ref = 1;
-    for my $l (@list) {
-        next if $l eq ''; #skip blank paths like /foo//bar/  (should just look up foo -> bar
-
-        my( $val, $ref ) = $self->_selectrow_array( "SELECT value, ref_id FROM field WHERE field=? AND obj_id=?",  $l, $next_ref );
-        die $self->{DBH}->errstr() if $self->{DBH}->errstr();
-
-        if( $ref ) {
-            $next_ref = $ref;
-        }
-        else {
-	    die "Unable to find xpath location '$path' for insert";
-        }
-    } #each path part
-
-    my( $field ) = $self->_selectrow_array( "SELECT max(field) + 1 FROM field WHERE obj_id=?", $next_ref );
-
-   if( index( $item_to_insert, 'v' ) == 0 ) {
-	$self->_do( "INSERT INTO field (obj_id,field,value) VALUES (?,?,?)", $next_ref, $field, substr( $item_to_insert, 1)  );
+    if( index( $val, 'v' ) == 0 ) {
+	$self->_do( "INSERT INTO field (obj_id,field,value) VALUES (?,?,?)", $list_id, $idx, substr( $val, 1 )  );
     } else {
-	$self->_do( "INSERT INTO field (obj_id,field,ref_id) VALUES (?,?,?)", $next_ref, $field, $item_to_insert );
+	$self->_do( "INSERT INTO field (obj_id,field,ref_id) VALUES (?,?,?)", $list_id, $idx, $val );
     }
+    return;
+} #list_insert
 
-} #xpath_list_insert
+sub hash_delete {
+    my( $self, $hash_id, $key ) = @_;
+    $self->_do( "DELETE FROM field WHERE obj_id=? AND field=?", $hash_id, $key );
+    return;
+}
 
+sub list_delete {
+    my( $self, $list_id, $idx ) = @_;
+    $self->_do( "DELETE FROM field WHERE obj_id=? AND field=?", $list_id, $idx );
+    return;
+}
+
+sub hash_insert {
+    my( $self, $hash_id, $key, $val ) = @_;
+    if( index( $val, 'v' ) == 0 ) {
+	$self->_do( "INSERT INTO field (obj_id,field,value) VALUES (?,?,?)", $hash_id, $key, substr( $val, 1 )  );
+    } else {
+	$self->_do( "INSERT INTO field (obj_id,field,ref_id) VALUES (?,?,?)", $hash_id, $key, $val );
+    }
+    die $self->{DBH}->errstr() if $self->{DBH}->errstr();
+    return;
+} #hash_insert
+
+sub list_fetch {
+    my( $self, $list_id, $idx ) = @_;
+    my( $val, $ref_id ) = $self->_selectrow_array( "SELECT value, ref_id FROM field WHERE obj_id=? AND field=?", $list_id, $idx );
+    return $ref_id || "v$val";
+} 
+
+sub hash_fetch {
+    my( $self, $hash_id, $key ) = @_;
+    my( $val, $ref_id ) = $self->_selectrow_array( "SELECT value, ref_id FROM field WHERE obj_id=? AND field=?", $hash_id, $key );
+    return $ref_id || "v$val";
+} 
+
+sub hash_has_key {
+    my( $self, $hash_id, $key ) = @_;
+    my( $fld ) = $self->_selectrow_array( "SELECT field FROM field WHERE obj_id=? AND field=?", $hash_id, $key );
+    return defined( $fld );
+}
 
 # ------------------------------------------------------------------------------------------
 #      * PRIVATE METHODS *
@@ -578,28 +426,6 @@ sub _selectall_arrayref {
     return $self->{DBH}->selectall_arrayref( $query, {}, @params );
 } #_selectall_arrayref
 
-
-sub _xpath_to_list {
-    my $path = shift;
-    my( @path ) = split( //, $path );
-    my( $working, $escaped, @res ) = '';
-    for my $ch (@path) {
-	if( $ch eq '/' && ! $escaped ) {
-	    push( @res, $working );
-	    $working = '';
-	    $escaped = 0;
-	} 
-	elsif( $ch eq '\\' ) {
-	    $escaped = 1;
-	}
-	else {
-	    $working .= $ch;
-	}
-    }
-    push( @res, $working ) if defined( $working );
-    return @res;
-} #_xpath_to_list
-
 #
 # Stores the object to persistance. Object is an array ref in the form id,class,data
 #
@@ -667,6 +493,10 @@ Yote::ObjProvider::init( datastore => 'Yote::SQLiteIO', db => 'yote_db', uname =
 
 =item commit_transaction( )
 
+=item count( container_id )
+
+returns the number of items in the given container
+
 =item database( )
 
 Provides a database handle. Used only in testing.
@@ -693,27 +523,37 @@ Returns the id for the given hash ref, array ref or yote object. If the argument
 
 Returns true if the object specified by the id can trace a path back to the root yote object.
 
+=item hash_delete( hash_id, key )
+
+Removes the key from the hash given by the id
+
+=item hash_fetch( hash_id, key )
+
+=item hash_has_key( hash_id, key )
+
+=item hash_insert( hash_id, key, value )
+
+=item list_delete( list_id, idx )
+
+=item list_fetch( list_id, idx )
+
+=item list_insert( list_id, val, idx )
+
+Inserts the item into the list with an optional index. If not given, this inserts to the end of the list.
+
 =item max_id( ) 
 
 Returns the max ID in the yote system. Used for testing.
 
 =item new
 
-=item paginate_xpath( path, start, length )
+=item paginate_hash( hash_id, length, start )
 
-This method returns a paginated portion of an object that is attached to the xpath given, as internal yote values.
+Returns a paginated hash reference
 
-=item paginate_xpath_list( parth, start, length )
+=item paginate_list( list_id, length, start )
 
-This method returns a paginated portion of a list that is attached to the xpath given.
-
-=item path_to_root( object )
-
-Returns the xpath of the given object tracing back a path to the root. This is not guaranteed to be the shortest path to root.
-
-=item paths_to_root( object )
-
-Returns the a list of all valid xpaths of the given object tracing back a path to the root. 
+Returns a paginated list reference
 
 =item recycle_object( obj_id )
 
@@ -733,27 +573,6 @@ Stores the object of class class encoded in the internal data format into the da
 
 Stows all objects that are marked as dirty. This is called automatically by the application server and need not be explicitly called.
 
-=item xpath( path )
-
-Given a path designator, returns the object data at the end of it, starting in the root. The notation is /foo/bar/baz where foo, bar and baz are field names. 
-
-=item xpath_count( path )
-
-Given a path designator, returns the number of fields of the object at the end of it, starting in the root. The notation is /foo/bar/baz where foo, bar and baz are field names. This is useful for counting how many things are in a list.
-
-=item xpath_delete( path )
-
-Deletes the entry specified by the path.
-
-=item xpath_insert( path, item )
-
-Inserts the item at the given xpath, overwriting anything that had existed previously.
-
-=item xpath_list_insert( path, item )
-
-Appends the item to the list located at the given xpath.
-
-
 =back
 
 =head1 AUTHOR
